"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const injected_scripts_1 = __importDefault(require("../injected-scripts"));
const parseNavigatorPlugins_1 = __importDefault(require("./parseNavigatorPlugins"));
function pageOverrides(args, data) {
    const { codecs, chrome, polyfills, navigator } = data;
    const injectedScripts = new injected_scripts_1.default();
    injectedScripts.add('errors');
    injectedScripts.add('navigator', {
        platform: args.platform,
        memory: args.memory,
        languages: args.languages,
        ensureOneVideoDevice: args.videoDevice,
    });
    injectedScripts.add('chrome', {
        updateLoadTimes: true,
        polyfill: {
            property: chrome.chrome,
            prevProperty: chrome.prevProperty,
        },
    });
    if (polyfills) {
        injectedScripts.add('polyfill', polyfills);
    }
    injectedScripts.add('plugins', parseNavigatorPlugins_1.default(navigator.navigator));
    injectedScripts.add('webGl');
    injectedScripts.add('console');
    injectedScripts.add('iframe');
    injectedScripts.add('shadowDom');
    if (args.windowFrame) {
        injectedScripts.add('screen', {
            windowFrame: args.windowFrame,
        });
    }
    const osString = `${args.osFamily} ${args.osVersion}`;
    let matchingCodec = codecs.find(x => x.opSyses.includes(osString));
    if (!matchingCodec) {
        // just match on os
        matchingCodec = codecs.find(x => x.opSyses.some(y => y.includes(args.osFamily)));
    }
    if (matchingCodec) {
        injectedScripts.add('codecs', {
            audioCodecs: matchingCodec.profile.audioSupport,
            videoCodecs: matchingCodec.profile.videoSupport,
        });
        injectedScripts.add('webRtc', {
            video: matchingCodec.profile.webRtcVideoCodecs,
            audio: matchingCodec.profile.webRtcAudioCodecs,
        });
    }
    return injectedScripts.build();
}
exports.default = pageOverrides;
//# sourceMappingURL=chromePageOverrides.js.map