"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = __importDefault(require("os"));
const fs_1 = __importDefault(require("fs"));
const platform = os_1.default.platform();
function readPolyfills(dirname) {
    let platformDir = '';
    if (platform === 'darwin') {
        platformDir = 'mac';
    }
    else if (platform === 'win32') {
        platformDir = 'windows';
    }
    else if (platform === 'linux') {
        platformDir = 'linux';
    }
    const polyfillsDir = `${dirname}/${platformDir}`;
    if (!platformDir || !fs_1.default.existsSync(polyfillsDir)) {
        return null;
    }
    const polyFills = {
        get(plugin) {
            const userAgent = plugin.userAgent;
            if (userAgent.os.family === 'Windows') {
                if (userAgent.os.major === '10') {
                    return this.windows10;
                }
                if (userAgent.os.major === '8') {
                    return this.windows8_1;
                }
                return this.windows7;
            }
            return this.mac;
        },
        canPolyfill(plugin) {
            return !!this.get(plugin);
        },
    };
    for (const filename of fs_1.default.readdirSync(polyfillsDir)) {
        if (!filename.startsWith('polyfill'))
            continue;
        const data = JSON.parse(fs_1.default.readFileSync(`${polyfillsDir}/${filename}`, 'utf8'));
        if (filename.includes('windows-7')) {
            polyFills.windows7 = data;
        }
        else if (filename.includes('windows-8')) {
            polyFills.windows8_1 = data;
        }
        else if (filename.includes('windows-10')) {
            polyFills.windows10 = data;
        }
        else if (filename.includes('mac')) {
            polyFills.mac = data;
        }
    }
    return polyFills;
}
exports.default = readPolyfills;
//# sourceMappingURL=readPolyfills.js.map